@@ -12,6 +12,9 @@ import binascii
 import warnings
 import logging
 
+class EpanLoadError(Exception):
+    pass
+
 try:
     from .epan2_ext import lib as epan_lib
     from .epan2_ext import ffi as epan_ffi
@@ -21,15 +24,23 @@ except ImportError:
         from .epan3_ext import lib as epan_lib
         from .epan3_ext import ffi as epan_ffi
         _epan_version = (3, 2)
-    except ImportError:
-        warnings.warn("Epan Library Extensions Not Found.")
-        raise EpanLoadError
+
+    except ImportError: #pragma: no cover
+
+        if os.getenv('RTD_AUTODOC_FAKE_EXT', None) is not None or \
+                os.getenv('READTHEDOCS', None) is not None:
+
+            warnings.warn("Epan Library Extensions Not Found. Loading fake_ext. This should only be useful when building docs on RTD.")
+
+            from .fake_ext import lib as epan_lib
+            from .fake_ext import ffi as epan_ffi
+
+            _epan_version = (2, 6)
+        else:
+            raise EpanLoadError
 
 _logger = logging.getLogger(__name__)
 
-class EpanLoadError(Exception):
-    pass
-
 _nstime_empty = epan_ffi.new('nstime_t *');
 @epan_ffi.callback('const nstime_t *(*)(struct packet_provider_data *prov, guint32 frame_num)')
 def _wishpy_get_ts(prov, frame_num):
@@ -92,6 +103,7 @@ def _epan_perform_one_packet_dissection_v2(wishpy_dissector, frames, hdr, packet
     elapsed_time_ptr = wishpy_dissector.elapsed_time_ptr
     ref_frame_data_ptr = wishpy_dissector.ref_frame_data_ptr
     last_frame_data = wishpy_dissector.last_frame_data
+    dfilter_obj_ptr = wishpy_dissector.dfilter_obj_ptr
 
     if ref_frame_data_ptr[0] == epan_ffi.NULL:
         curr_frame_data = wishpy_dissector.first_frame_data
@@ -139,7 +151,10 @@ def _epan_perform_one_packet_dissection_v2(wishpy_dissector, frames, hdr, packet
             elapsed_time_ptr, ref_frame_data_ptr, last_frame_data)
 
     # Not sure what this is - Just copied from `tshark` sources
-    epan_lib.prime_epan_dissect_with_postdissector_wanted_hfids(epan_dissect_obj)
+    # epan_lib.prime_epan_dissect_with_postdissector_wanted_hfids(epan_dissect_obj)
+
+    if dfilter_obj_ptr:
+        epan_lib.epan_dissect_prime_with_dfilter(epan_dissect_obj, dfilter_obj_ptr)
 
     ## Do actual dissection here.
     # Get buffer and tvbuff first and then run dissector
@@ -147,11 +162,17 @@ def _epan_perform_one_packet_dissection_v2(wishpy_dissector, frames, hdr, packet
     epan_lib.epan_dissect_run(epan_dissect_obj, wth_file_type, rec,
             tvb_ptr, curr_frame_data, epan_ffi.NULL)
 
-    dissected = cb_func(epan_dissect_obj)
+    filter_passed = True
+    if dfilter_obj_ptr:
+        filter_passed = epan_lib.dfilter_apply_edt(dfilter_obj_ptr, epan_dissect_obj)
 
-    # Get into last data - useful for relative analysis
-    epan_ffi.memmove(last_frame_data,
-            curr_frame_data, epan_ffi.sizeof('frame_data'))
+    dissected = None
+    if filter_passed :
+        dissected = cb_func(epan_dissect_obj)
+
+        # Get into last data - useful for relative analysis
+        epan_ffi.memmove(last_frame_data,
+                curr_frame_data, epan_ffi.sizeof('frame_data'))
 
     # Reset the frame data and dissector object
     epan_lib.frame_data_set_after_dissect(curr_frame_data, cum_bytes)
@@ -169,6 +190,7 @@ def _epan_perform_dissection_v2(wishpy_dissector, wth, wth_file_type, cb_func, c
     elapsed_time_ptr = wishpy_dissector.elapsed_time_ptr
     ref_frame_data_ptr = wishpy_dissector.ref_frame_data_ptr
     last_frame_data = wishpy_dissector.last_frame_data
+    dfilter_obj_ptr = wishpy_dissector.dfilter_obj_ptr
 
     offset = epan_ffi.new('gint64 *')
 
@@ -207,9 +229,12 @@ def _epan_perform_dissection_v2(wishpy_dissector, wth, wth_file_type, cb_func, c
                     elapsed_time_ptr, ref_frame_data_ptr, last_frame_data)
 
             # Not sure what this is - Just copied from `tshark` sources
-            epan_lib.prime_epan_dissect_with_postdissector_wanted_hfids(
-                    epan_dissect_obj)
+            #epan_lib.prime_epan_dissect_with_postdissector_wanted_hfids(
+                    #epan_dissect_obj)
 
+            if dfilter_obj_ptr:
+                epan_lib.epan_dissect_prime_with_dfilter(
+                        epan_dissect_obj, dfilter_obj_ptr)
             ## Do actual dissection here.
             # Get buffer and tvbuff first and then run dissector
             tvb_ptr = epan_lib.tvb_new_real_data(buf_ptr, pkt_len,
@@ -222,11 +247,18 @@ def _epan_perform_dissection_v2(wishpy_dissector, wth, wth_file_type, cb_func, c
                 skipped += 1
                 continue
 
-            dissected = cb_func(epan_dissect_obj)
+            filter_passed = True
+            if dfilter_obj_ptr:
+                filter_passed = epan_lib.dfilter_apply_edt(dfilter_obj_ptr,
+                        epan_dissect_obj)
 
-            # Get into last data - useful for relative analysis
-            epan_ffi.memmove(last_frame_data,
-                curr_frame_data, epan_ffi.sizeof('frame_data'))
+            dissected = None
+            if filter_passed:
+                dissected = cb_func(epan_dissect_obj)
+
+                # Get into last data - useful for relative analysis
+                epan_ffi.memmove(last_frame_data,
+                    curr_frame_data, epan_ffi.sizeof('frame_data'))
 
             # Reset the frame data and dissector object
             epan_lib.frame_data_set_after_dissect(curr_frame_data, cum_bytes)
@@ -234,7 +266,8 @@ def _epan_perform_dissection_v2(wishpy_dissector, wth, wth_file_type, cb_func, c
 
             processed += 1
 
-            yield dissected
+            if filter_passed:
+                yield dissected
 
             if processed == count + 1:
                 break
@@ -255,6 +288,7 @@ def _epan_perform_one_packet_dissection_v3(wishpy_dissector, frames, hdr, packet
     elapsed_time_ptr = wishpy_dissector.elapsed_time_ptr
     ref_frame_data_ptr = wishpy_dissector.ref_frame_data_ptr
     last_frame_data = wishpy_dissector.last_frame_data
+    dfilter_obj_ptr = wishpy_dissector.dfilter_obj_ptr
 
     if ref_frame_data_ptr[0] == epan_ffi.NULL:
         curr_frame_data = wishpy_dissector.first_frame_data
@@ -304,8 +338,11 @@ def _epan_perform_one_packet_dissection_v3(wishpy_dissector, frames, hdr, packet
             elapsed_time_ptr, ref_frame_data_ptr, last_frame_data)
 
     # Not sure what this is - Just copied from `tshark` sources
-    epan_lib.prime_epan_dissect_with_postdissector_wanted_hfids(
-            epan_dissect_obj)
+    #epan_lib.prime_epan_dissect_with_postdissector_wanted_hfids(
+            #epan_dissect_obj)
+
+    if dfilter_obj_ptr:
+        epan_lib.epan_dissect_prime_with_dfilter(epan_dissect_obj, dfilter_obj_ptr)
 
     ## Ready to do `dissection` here.
     # Get buffer and tvbuff first and then run dissector
@@ -315,11 +352,18 @@ def _epan_perform_one_packet_dissection_v3(wishpy_dissector, frames, hdr, packet
     epan_lib.epan_dissect_run(epan_dissect_obj, wth_file_type,
             rec, tvb_ptr, curr_frame_data, epan_ffi.NULL)
 
-    dissected = cb_func(epan_dissect_obj)
+    filter_passed = True
+    if dfilter_obj_ptr is not None:
+        filter_passed = epan_lib.dfilter_apply_edt(dfilter_obj_ptr, epan_dissect_obj)
+
+    dissected = None
+    if filter_passed:
+        dissected = cb_func(epan_dissect_obj)
+
+        # Get into last data - useful for relative analysis
+        epan_ffi.memmove(last_frame_data,
+                curr_frame_data, epan_ffi.sizeof('frame_data'))
 
-    # Get into last data - useful for relative analysis
-    epan_ffi.memmove(last_frame_data,
-            curr_frame_data, epan_ffi.sizeof('frame_data'))
     # Reset the frame data and dissector object
     epan_lib.frame_data_set_after_dissect(curr_frame_data, cum_bytes)
     epan_lib.epan_dissect_reset(epan_dissect_obj)
@@ -338,6 +382,7 @@ def _epan_perform_dissection_v3(wishpy_dissector, wth, wth_file_type, cb_func, c
     elapsed_time_ptr = wishpy_dissector.elapsed_time_ptr
     ref_frame_data_ptr = wishpy_dissector.ref_frame_data_ptr
     last_frame_data = wishpy_dissector.last_frame_data
+    dfilter_obj_ptr = wishpy_dissector.dfilter_obj_ptr
 
     offset = epan_ffi.new('gint64 *')
 
@@ -376,9 +421,12 @@ def _epan_perform_dissection_v3(wishpy_dissector, wth, wth_file_type, cb_func, c
                     elapsed_time_ptr, ref_frame_data_ptr, last_frame_data)
 
             # Not sure what this is - Just copied from `tshark` sources
-            epan_lib.prime_epan_dissect_with_postdissector_wanted_hfids(
-                    epan_dissect_obj)
+            #epan_lib.prime_epan_dissect_with_postdissector_wanted_hfids(
+                    #epan_dissect_obj)
 
+            if dfilter_obj_ptr:
+                epan_lib.epan_dissect_prime_with_dfilter(
+                        epan_dissect_obj, dfilter_obj_ptr)
             ## Do actual dissection here.
             # Get buffer and tvbuff first and then run dissector
             tvb_ptr = epan_lib.tvb_new_real_data(buf[0].data, pkt_len,
@@ -391,11 +439,16 @@ def _epan_perform_dissection_v3(wishpy_dissector, wth, wth_file_type, cb_func, c
                 skipped += 1
                 continue
 
-            dissected = cb_func(epan_dissect_obj)
+            filter_passed = True
+            if dfilter_obj_ptr is not None:
+                filter_passed = epan_lib.dfilter_apply_edt(dfilter_obj_ptr,
+                        epan_dissect_obj)
+            if filter_passed :
+                dissected = cb_func(epan_dissect_obj)
 
-            # Get into last data - useful for relative analysis
-            epan_ffi.memmove(last_frame_data,
-                    curr_frame_data, epan_ffi.sizeof('frame_data'))
+                # Get into last data - useful for relative analysis
+                epan_ffi.memmove(last_frame_data,
+                        curr_frame_data, epan_ffi.sizeof('frame_data'))
 
             # Reset the frame data and dissector object
             epan_lib.frame_data_set_after_dissect(curr_frame_data, cum_bytes)
@@ -403,7 +456,8 @@ def _epan_perform_dissection_v3(wishpy_dissector, wth, wth_file_type, cb_func, c
 
             processed += 1
 
-            yield dissected
+            if filter_passed:
+                yield dissected
 
             if processed == count + 1:
                 break
