@@ -14,6 +14,7 @@ Example:
     print(packet)
 
 """
+import os
 import socket
 import struct
 import json
@@ -22,8 +23,17 @@ from datetime import datetime as dt
 import unicodedata
 import logging
 
+_logger = logging.getLogger(__name__)
+
 from ._wrapper import *
-from ...libpcap.lib.capturer import PCAPHeader, pcap_ffi
+try:
+    from ...libpcap.lib.capturer import PCAPHeader, pcap_ffi
+except: #pragma: no cover
+    if os.getenv('READTHEDOCS', None) is not None:
+        _logger.warning("Import Error, but it's okay during RTD Build.")
+    else:
+        _logger.warning("Import Error, Ignoring for now.")
+        pass #raise
 
 class WishpyErrorInitDissector(Exception):
     """Error raised during initialization of dissector and or dissector session.
@@ -45,8 +55,6 @@ class WishpyErrorWthOpen(Exception):
     """
     pass
 
-_logger = logging.getLogger(__name__)
-
 _EPAN_LIB_INITIALIZED = False
 
 
@@ -56,8 +64,9 @@ class WishpyDissectorBase:
     """
 
     _pretty = False
-
-    _test_json = False
+    _add_proto_tree = False
+    _test_json = True
+    _elasticky = False
 
     # We are having these definitions in the class because a lot many of them
     # are used in fast path and having them dereferenced here gives us some
@@ -119,8 +128,6 @@ class WishpyDissectorBase:
     FT_FLOAT = epan_lib.FT_FLOAT
     FT_DOUBLE = epan_lib.FT_DOUBLE
 
-
-
     epan_int_types = [
                 FT_INT8,
                 FT_INT16,
@@ -146,17 +153,17 @@ class WishpyDissectorBase:
     unquoted_types = epan_all_int_types + \
             [FT_BOOLEAN, FT_RELATIVE_TIME, FT_FLOAT, FT_DOUBLE]
 
-    # Below are some dict's required for printing few packet types
+    # Keys: BAS_XXX value `quote` (Whether returned value should be quoted.)
     hfbases = {
-            BASE_NONE : ('{:d}', False), # Not sure how this is to be treated
-            BASE_DEC : ('{:d}', False),
-            BASE_HEX : ('0x{:x}', True),
-            BASE_OCT : ('{:o}', True),
-            BASE_DEC_HEX: ('{:d}(0x{:x})', True),
-            BASE_HEX_DEC: ('0x{:x}({:d})', True),
-            BASE_PT_TCP: ('{:d}', False),
-            BASE_PT_UDP: ('{:d}', False),
-            BASE_PT_SCTP: ('{:d}', False)
+            BASE_NONE : False, # Not sure how this is to be treated
+            BASE_DEC : False,
+            BASE_HEX : True,
+            BASE_OCT : True,
+            BASE_DEC_HEX: True,
+            BASE_HEX_DEC: True,
+            BASE_PT_TCP: False,
+            BASE_PT_UDP: False,
+            BASE_PT_SCTP: False
 
      }
 
@@ -164,202 +171,36 @@ class WishpyDissectorBase:
     fvalue_to_string_repr = epan_lib.fvalue_to_string_repr
     wmem_free = epan_lib.wmem_free
 
+    @classmethod
+    def set_elasticky(cls, enabled):
+        cls._elasticky = enabled
+
     @classmethod
     def enable_json_test(cls):
         cls._test_json = True
 
     @classmethod
-    def enable_pretty_print(cls):
-        self._pretty = True
-
-    @classmethod
-    def remove_ctrl_chars(cls, s):
-        """Removes the Ctrl Characters from the string.
-        """
-        # FIXME: May be we should replace them with their unicode code points
-        category_fn = unicodedata.category
-        return "".join(ch for ch in s if category_fn(ch)[0] != "C")
-
-    @classmethod
-    def func_not_supported(cls, *args):
-        return "Not Supported"
-
-    @classmethod
-    def epan_ether_to_str(cls, fvalue, ftype, display, abbrev):
-        """ Converting Ethernet addresses to String"""
-
-        eth_bytes = fvalue.value.bytes
-
-        display_bytes = []
-        for i in range(eth_bytes.len):
-            display_byte = "{:02X}".format(eth_bytes.data[i])
-            display_bytes.append(display_byte)
-
-        return ":".join(display_bytes), True
-
-    epan_bytes_to_str = epan_ether_to_str
-
-    @classmethod
-    def epan_str_to_str(cls, fvalue, ftype, display, abbrev):
-        """Converting epan 'char *' to Python String"""
-
-        value = fvalue.value.string
-
-        # If display is BASE_NONE, utf-8 is fine, but for others, not always! (GSM-Unicode)
-        if display == 0:
-            x = cls.epan_string(value).decode().\
-                    replace('\\', '\\\\').\
-                    replace('"', '\\"')
-            return cls.remove_ctrl_chars(x), True
+    def set_pretty_print_details(cls, enabled=False, add_proto_tree=False):
+        cls._pretty = enabled
+        if enabled:
+            cls.packet_print_func = cls.print_dissected_tree_json_pretty
         else:
-            try:
-                x = cls.epan_string(value).decode("utf-8").\
-                        replace('\\', '\\\\').\
-                        replace('"', '\\"')
-                return cls.remove_ctrl_chars(x), True
-            except Exception as e:
-                _logger.exception("epan_str_to_str: (%s) %s", abbrev, x)
-                return "Cannot Decode"
+            cls.packet_print_func = cls.print_dissected_tree_json
+
+        if add_proto_tree:
+            cls._add_proto_tree = True
 
     @classmethod
-    def epan_ipv4_to_str(cls, fvalue, ftype, display, abbrev):
-        """Converting IP Address to Python String"""
-
-        ipv4 = fvalue.value.ipv4
-
-        return socket.inet_ntoa(struct.pack('!I', ipv4.addr)), True
+    def print_dissected_tree_json_pretty(cls, dissector):
+        node = dissector[0].tree
+        return cls.print_dissected_tree_json_node_pretty(node)
 
     @classmethod
-    def epan_bool_to_str(cls, fvalue, ftype, display, abbrev,
-            on_off=False, json_compat=True):
-        """ Converting `gboolean` to String"""
-
-        if on_off and json_compat:
-            raise ValueError("Specify Either on_off or json_compat not both.")
-
-        value = bool(fvalue.value.uinteger)
-
-        if value:
-            if json_compat:
-                return "true", False
-            if on_off:
-                return "ON", True
-        else:
-            if json_compat:
-                return "false", False
-            if on_off:
-                return "OFF", True
-
-        return "{}".format(value)
-
-    @classmethod
-    def epan_int_to_str(cls, fvalue, ftype, display, abbrev):
-        """Converting Integer to String, using BASE_* property."""
-
-        try:
-            # FIXME: We can definitely do better than these `if`s.
-            # We are ignoring all fancy display options for now
-            if display & cls.BASE_RANGE_STRING:
-                display ^= cls.BASE_RANGE_STRING
-
-            if display & cls.BASE_EXT_STRING:
-                display ^= cls.BASE_EXT_STRING
-
-            if display & cls.BASE_VAL64_STRING:
-                display ^= cls.BASE_VAL64_STRING
-
-            if display & cls.BASE_ALLOW_ZERO:
-                display ^= cls.BASE_ALLOW_ZERO
-
-            if display & cls.BASE_UNIT_STRING:
-                display ^= cls.BASE_UNIT_STRING
-
-            if display & cls.BASE_NO_DISPLAY_VALUE:
-                display ^= cls.BASE_NO_DISPLAY_VALUE
-
-            if display & cls.BASE_PROTOCOL_INFO:
-                display ^= cls.BASE_PROTOCOL_INFO
-
-            if display & cls.BASE_SPECIAL_VALS:
-                display ^= cls.BASE_SPECIAL_VALS
-
-            # Change the custom display to `Decimal` for now
-            if display == cls.BASE_CUSTOM:
-                display = cls.BASE_DEC
-
-            if display == cls.BASE_OUI:
-                display = cls.BASE_HEX
-
-            base_format, quote = cls.hfbases[display]
-
-        except:
-            _logger.exception("epan_int_to_str: (%s) %d %d", abbrev, ftype, display)
-            return "type: {} display: {} Not Supported".format(ftype, display), True
-
-        return base_format.format(fvalue.value.uinteger,
-                fvalue.value.uinteger), quote
-
-    @classmethod
-    def epan_abstime_to_str(cls, fvalue, ftype, display, abbrev):
-        timeval = fvalue.value.time
-        timeval = timeval.secs + timeval.nsecs / 1000000000
-        value = dt.strftime(dt.fromtimestamp(timeval), '%d-%b-%Y %H:%M:%S.%f %Z')
-        return value, True
-
-
-    @classmethod
-    def epan_reltime_to_str(cls, fvalue, ftype, display, abbrev):
-        value = fvalue.value.time
-        return "{:.9f}".format(value.secs + value.nsecs / 1000000000), False
-
-    @classmethod
-    def epan_none_to_str(cls, fvalue, ftype, display, abbrev):
-        return None, None
-
-    to_str_funcs = {
-            FT_ETHER : epan_ether_to_str.__func__,
-            FT_IPv4 : epan_ipv4_to_str.__func__,
-            FT_BOOLEAN : epan_bool_to_str.__func__,
-            FT_STRING : epan_str_to_str.__func__,
-            FT_BYTES : epan_bytes_to_str.__func__,
-            FT_RELATIVE_TIME : epan_reltime_to_str.__func__,
-            FT_ABSOLUTE_TIME : epan_abstime_to_str.__func__,
-            FT_NONE : epan_none_to_str.__func__,
-            FT_PROTOCOL : epan_none_to_str.__func__
-    }
-
-    all_int_to_str_funcs = [epan_int_to_str.__func__] * len(epan_all_int_types)
-    to_str_funcs.update(zip(epan_all_int_types, all_int_to_str_funcs))
-
-    @classmethod
-    def value_to_str(cls, finfo):
-        """
-        Returns string representation of the `finfo.value`
-        """
-
-        fvalue = finfo.value
-        ftype = finfo.hfinfo[0].type
-        display = finfo.hfinfo[0].display
-        abbrev = cls.epan_string(finfo.hfinfo[0].abbrev).decode()
-
-        epan_int_types = cls.epan_int_types
-        epan_uint32_types = cls.epan_uint32_types
-        epan_uint_types = cls.epan_uint_types
-        epan_all_int_types = cls.epan_all_int_types
-
-        try:
-            fn = cls.to_str_funcs[ftype]
-            return fn(cls, fvalue, ftype, display, abbrev)
-        except KeyError as e:
-            #_logger.exception("unknown type: %d", ftype)
-
-            return "{} {}".format(ftype, display), True
-
-    @classmethod
-    def print_dissected_tree_pretty_ftype_api(cls, node_ptr, level=1):
+    def print_dissected_tree_json_node_pretty(cls, node_ptr, level=1):
         """Returns a string that represents a dissected tree.
         """
         return_str = ""
+        tabstop = 4
 
         node = node_ptr[0]
         finfo = node.finfo
@@ -384,11 +225,11 @@ class WishpyDissectorBase:
                     if hfinfo.type == cls.FT_STRING:
                         finfo_display_str = finfo_display_str.\
                                 replace('\\', '\\\\').replace('"', '\\"')
-                        finfo_display_str = cls.remove_ctrl_chars(finfo_display_str)
+                        finfo_display_str = cls._remove_ctrl_chars(finfo_display_str)
                     finfo_display_str = '"' + finfo_display_str + '"'
                 else:
                     try:
-                        _, quote = cls.hfbases[display]
+                        quote = cls.hfbases[display]
                     except KeyError as e:
                         quote = True
 
@@ -399,8 +240,8 @@ class WishpyDissectorBase:
 
                 return_str += finfo_display_str
 
-        lspaces = " " * level
-        lspaces_1 = " " * (level - 1)
+        lspaces = " " * level * tabstop
+        lspaces_1 = " " * (level - 1) * tabstop
         newlevel = level + 1
 
         child = node.first_child
@@ -419,11 +260,18 @@ class WishpyDissectorBase:
             return_str += "\n"
             while child != cls.NULL:
                 return_str += lspaces
-                return_str += cls.print_dissected_tree_pretty_ftype_api(child, newlevel)
+                return_str += cls.print_dissected_tree_json_node_pretty(child, newlevel)
                 child = child.next
             return_str += lspaces
 
             return_str += "\n" + lspaces_1
+
+            # details - protect by a flag
+            if level == 1 and cls._add_proto_tree:
+                return_str += ","
+                return_str += "\"proto_tree\":"
+                return_str += "\"" + cls.print_dissected_tree_details_node(node_ptr, level) + "\""
+
             return_str += "}"
         else: # child is not None So we have someone who's FT_NONE, FT_PROTOCOL and no tree?
             if not finfo_display_str:
@@ -434,55 +282,127 @@ class WishpyDissectorBase:
         return return_str
 
     @classmethod
-    def print_dissected_tree(cls, node_ptr):
-        """Returns a string that represents a dissected tree.
-        """
-        return_str = ""
+    def print_dissected_tree_json(cls, dissector):
+        node = dissector[0].tree
+        return cls.print_dissected_tree_json_node_simple(node)
 
+    @classmethod
+    def print_dissected_tree_json_node_simple(cls, node_ptr, level=1, add_subtrees=True):
+
+        tabstop = 2
+
+        if cls._elasticky:
+            add_subtrees = False
+
+        inside_subtree = False
+
+        pretty = False
+        spaces = ""
+        if pretty:
+            spaces = " " * tabstop * (level - 1)
+
+        return_str = ""
         node = node_ptr[0]
         finfo = node.finfo
+        hfinfo = None
+
         if finfo != cls.NULL:
 
             hfinfo = finfo.hfinfo[0]
             abbrev = cls.epan_string(hfinfo.abbrev).decode()
-            abbrev_str = '"' + abbrev + '"'
-            return_str += abbrev_str + ":"
-            finfo_display_str, quote = cls.value_to_str(finfo)
-            if finfo_display_str:
-                if quote:
-                    finfo_display_str = '"' + finfo_display_str + '"'
-        else:
-            finfo_display_str = ""
+            display = hfinfo.display
 
-        if finfo_display_str is not None:
-            return_str += finfo_display_str
+            finfo_str = cls.fvalue_to_string_repr(
+                    cls.NULL,
+                    cls.addressof(finfo[0].value),
+                    cls.FTREPR_DISPLAY,
+                    display)
+            #print(abbrev, hfinfo.type, finfo_str, level)
+
+            abbrev_str = spaces + "\"" + abbrev + "\":"
+            if hfinfo.type == 0:
+                # FIXME: Properly add a tree here
+                if not add_subtrees:
+
+                    field_str = "\"\""
+                    if node.first_child != cls.NULL:
+                        field_str += ","
+                else:
+
+                    field_str = "{"
+                    inside_subtree = True
+                    if pretty:
+                        field_str += "\n"
+
+                return_str += abbrev_str + field_str
+            elif hfinfo.type == 1:
+
+                # protocol:
+                field_str = "{"
+                inside_subtree = True
+                if pretty:
+                    field_str += "\n"
+
+                return_str += abbrev_str + field_str
+            else:
+                if finfo_str != cls.NULL:
+                    field_str = cls.epan_string(finfo_str).decode()
+                    if hfinfo.type not in cls.unquoted_types:
+                        if hfinfo.type == cls.FT_STRING:
+                            field_str = field_str.\
+                                    replace('\\', '\\\\').replace('"', '\\"')
+                            field_str = cls._remove_ctrl_chars(field_str)
+                        field_str = '"' + field_str + '"'
+                    else:
+                        try:
+                            quote = cls.hfbases[display]
+                        except KeyError as e:
+                            quote = True
+
+                        if quote:
+                            field_str = '"' + field_str + '"'
+
+                else:
+                    field_str = ""
+
+                # If this field has a child, we've to add it's "," now
+                if node.first_child != cls.NULL:
+                    field_str += ","
+                    if pretty:
+                        field_str += "\n"
+
+                return_str += abbrev_str + field_str
+                    # FIXME: We are going to add tree here!!
+        else:
+            # Top Level
+            return_str += "{"
 
         child = node.first_child
-        if child != cls.NULL:
 
-            if finfo_display_str:
+        while child != cls.NULL:
+            child_str = cls.print_dissected_tree_json_node_simple(child, level+1)
+            return_str += child_str
+
+            child = child.next
+
+        if finfo != cls.NULL:
+            if inside_subtree:
+                return_str += spaces + "}"
+        else:
+            # Top Level
+            return_str += "}"
+
+        if node.next != cls.NULL:
+            if return_str:
                 return_str += ","
 
-                abbrev_tree = abbrev + "_tree"
-                abbrev_tree_str = '"' + abbrev_tree + '"'
-                return_str += abbrev_tree_str + ":"
-
-            return_str += "{"
-            while child != cls.NULL:
-                return_str += cls.print_dissected_tree(child)
-                child = child.next
-
-            return_str += "}"
-        else: # child is not None So we have someone who's FT_NONE, FT_PROTOCOL and no tree?
-            if not finfo_display_str:
-                return_str += "\"\""
-        if node.next != cls.NULL:
-            return_str += ","
+        if pretty:
+            return_str += "\n"
 
         return return_str
 
     @classmethod
-    def print_dissected_tree_ftype_api(cls, node_ptr):
+    def print_dissected_tree_json_node(cls, node_ptr, level=1):
         """Returns a string representing dissected tree using the `ftypes` API.
         """
         return_str = ""
@@ -494,8 +414,9 @@ class WishpyDissectorBase:
         if finfo != cls.NULL:
 
             hfinfo = finfo.hfinfo[0]
-            display = hfinfo.display
             abbrev = cls.epan_string(hfinfo.abbrev).decode()
+            display = hfinfo.display
+
             abbrev_str = '"' + abbrev + '"'
             return_str += abbrev_str + ":"
             finfo_str = cls.fvalue_to_string_repr(
@@ -510,11 +431,11 @@ class WishpyDissectorBase:
                     if hfinfo.type == cls.FT_STRING:
                         finfo_display_str = finfo_display_str.\
                                 replace('\\', '\\\\').replace('"', '\\"')
-                        finfo_display_str = cls.remove_ctrl_chars(finfo_display_str)
+                        finfo_display_str = cls._remove_ctrl_chars(finfo_display_str)
                     finfo_display_str = '"' + finfo_display_str + '"'
                 else:
                     try:
-                        _, quote = cls.hfbases[display]
+                        quote = cls.hfbases[display]
                     except KeyError as e:
                         quote = True
 
@@ -525,6 +446,7 @@ class WishpyDissectorBase:
 
                 return_str += finfo_display_str
 
+        newlevel = level + 1
         child = node.first_child
         if child != cls.NULL:
 
@@ -537,9 +459,13 @@ class WishpyDissectorBase:
 
             return_str += "{"
             while child != cls.NULL:
-                return_str += cls.print_dissected_tree_ftype_api(child)
+                return_str += cls.print_dissected_tree_json_node(child, newlevel)
                 child = child.next
 
+            if level == 1 and cls._add_proto_tree:
+                return_str += ","
+                return_str += "\"proto_tree\":"
+                return_str += "\"" + cls.print_dissected_tree_details_node(node_ptr, level) + "\""
             return_str += "}"
         else: # child is not None So we have someone who's FT_NONE, FT_PROTOCOL and no tree?
             if not finfo_display_str:
@@ -549,22 +475,91 @@ class WishpyDissectorBase:
 
         return return_str
 
+    packet_print_func = print_dissected_tree_json
+
+    @classmethod
+    def print_dissected_tree_details_api(cls, dissector):
+        """Print a packets Protocol tree using `proto_tree_json` API
+        """
+        ops = epan_ffi.new('print_stream_ops_t *')
+        print_stream = new_wishpy_print_stream(ops)
+
+        cls.ops = ops
+        cls.print_stream = print_stream
+
+        epan_lib.proto_tree_print(
+                epan_lib.print_dissections_expanded,
+                False,
+                dissector,
+                epan_ffi.NULL,
+                print_stream)
+
+    @classmethod
+    def print_dissected_tree_details(cls, dissector):
+        """
+        """
+        node = dissector[0].tree
+        level = 0
+        return cls.print_dissected_tree_details_node(node, level)
+
+
+    @classmethod
+    def print_dissected_tree_details_node(cls, node_ptr, level):
+
+        tabstop = 4
+        label_str = epan_ffi.new('gchar [240]')
+        return_str = ""
+        spaces = (level - 1) * " " * tabstop
+
+        node = node_ptr[0]
+        finfo = node.finfo
+
+        finfo_display_str = None
+        if finfo != cls.NULL:
+
+            hfinfo = finfo.hfinfo[0]
+            abbrev = cls.epan_string(hfinfo.abbrev).decode()
+            display = hfinfo.display
+            rep = finfo.rep
+
+            abbrev_str = '"' + abbrev + '"'
+            #return_str += abbrev_str + ":"
+
+            if rep != epan_ffi.NULL:
+                finfo_str = epan_ffi.string(rep[0].representation)
+            else:
+                epan_lib.proto_item_fill_label(finfo, label_str)
+                finfo_str = epan_ffi.string(label_str)
+
+            finfo_display_str =finfo_str.decode()
+            finfo_display_str = finfo_display_str.\
+                    replace('\\', '\\\\').replace('"', '\\"')
+            finfo_display_str = spaces + cls._remove_ctrl_chars(finfo_display_str)
+
+            return_str += finfo_display_str + "\n"
+
+        newlevel = level + 1
+        child = node.first_child
+        if child != cls.NULL:
+
+            while child != cls.NULL:
+                return_str += cls.print_dissected_tree_details_node(child, newlevel)
+                child = child.next
+
+
+        return return_str
+
     @classmethod
     def packet_to_json(cls, handle_ptr):
         """ An example method that depicts how to use internal dissector API."""
 
-        dissector = handle_ptr[0]
-
-        # FIXME: following should be like json dumps
-        if cls._pretty:
-            s = cls.print_dissected_tree_pretty_ftype_api(dissector.tree)
-        else:
-            s = cls.print_dissected_tree_ftype_api(dissector.tree)
+        s = cls.packet_print_func(handle_ptr)
         try:
-            if cls._test_json:
-                _ = json.loads(s, strict=False)
+            if cls._elasticky:
+                x = json.loads(s, strict=False, object_pairs_hook=cls.get_elasticky_json)
+                s = json.dumps(x)
         except json.decoder.JSONDecodeError as e:
-            _logger.exception("packet_to_json", e.doc)
+            _logger.exception("packet_to_json %s", e.doc)
             return {}
         except Exception as e:
             _logger.exception("packet_to_json")
@@ -573,6 +568,33 @@ class WishpyDissectorBase:
 
         return s
 
+    @classmethod
+    def get_elasticky_json(cls, values):
+
+        return_dict = {}
+        for k,v in values:
+            dict_key = k.replace(".", "_")
+            if dict_key in return_dict:
+                old = return_dict.pop(dict_key)
+                if isinstance(old, list):
+                    new = [v] + old
+                else:
+                    new = [old, v]
+                return_dict[dict_key] = new
+            else:
+                return_dict[dict_key] = v
+
+        return dict(return_dict)
+
+
+    @classmethod
+    def _remove_ctrl_chars(cls, s):
+        """Removes the Ctrl Characters from the string.
+        """
+        # FIXME: May be we should replace them with their unicode code points
+        category_fn = unicodedata.category
+        return "".join(ch for ch in s if category_fn(ch)[0] != "C")
+
     def __init__(self, *args, **kw):
         self._epan_dissector = None
         self._elapsed_time_ptr = None
@@ -580,6 +602,7 @@ class WishpyDissectorBase:
         self._first_frame_data = None
         self._last_frame_data = None
         self._provider = None
+        self._dfilter_obj_ptr = None
 
     @property
     def last_frame_data(self):
@@ -602,6 +625,10 @@ class WishpyDissectorBase:
 
         return self._epan_dissector
 
+    @property
+    def dfilter_obj_ptr(self):
+        return self._dfilter_obj_ptr
+
     @property
     def epan_session(self):
         if self._epan_dissector is None:
@@ -647,12 +674,14 @@ class WishpyDissectorBase:
         del self._last_frame_data
         del self._provider
         del self._ref_frame_data_ptr
+        del self._dfilter_obj_ptr
 
         self._elapsed_time_ptr = None
 
 
     def run(self, *args, **kw):
-        """A generator function `yield`ing at-least the dissected packets.
+        """A generator function ``yield``ing at\-least the dissected packets.
+
 
         Implementing this as a generator function helps one to run code
         that looks like
@@ -666,6 +695,52 @@ class WishpyDissectorBase:
         """
         raise NotImplemented("Derived Classes need to implement this.")
 
+    def apply_filter(self, filter_str, overwrite=False):
+        """
+        Applies a filter given by the filter_str to the dissection.
+
+        Args:
+            filter_str: str - A string that is in a wireshark filter format.
+
+        Returns:
+            result: (int, str) - Result of application 0 success. Negative value
+            suggesting error. Caller should check the error.
+
+        Note: Right now it is recommended to run this method before `run` method
+        is called on the dissector.
+
+        """
+        if self._dfilter_obj_ptr and not overwrite:
+            return (-1, "Display Filter already set and 'overwrite' not set.")
+
+        dfilter_obj_ptr = epan_ffi.new('dfilter_t **')
+        err_str_ptr = epan_ffi.new('gchar **')
+        result = epan_lib.dfilter_compile(
+                filter_str.encode(),
+                dfilter_obj_ptr,
+                err_str_ptr)
+
+        if not result:
+            err_str = epan_ffi.string(err_str_ptr[0])
+            return (-2, err_str.decode())
+
+        old = None
+        if self._dfilter_obj_ptr:
+            old = self._dfilter_obj_ptr
+
+        self._dfilter_obj_ptr = dfilter_obj_ptr[0]
+        if old is not None:
+            del old_dfilter_obj_ptr
+
+        return (0, None)
+
+    def clear_filter(self):
+        """Clears the dfilter if any."""
+        if self._dfilter_obj_ptr:
+            o = self._dfilter_obj_ptr
+            self._dfilter_obj_ptr = None
+            del o
+
 
 class WishpyDissectorFile(WishpyDissectorBase):
     """Dissector class for PCAP Files.
@@ -676,10 +751,10 @@ class WishpyDissectorFile(WishpyDissectorBase):
         self.__filename = filename
 
     def run(self, count=0, skip=-1):
-        """
-        Actual function that performs the Dissection. Right now since we are
-        only supporting dissecting packets from Wiretap supported files,
-        only dissects packets from a pcap(ish) file.
+        """Actual function that performs the Dissection.
+
+        Right now since we are only supporting dissecting packets from Wiretap
+        supported files, only dissects packets from a pcap(ish) file.
 
         """
 
@@ -726,13 +801,17 @@ class WishpyDissectorQueue(WishpyDissectorBase):
     def dissect_one_packet(self):
         """Dissects a single packet
 
-        This should typically call `fetch` and the perform dissection. All
-        the queue based `dissectors` will dissect one packet at a time, so
+        This should typically call ``fetch`` and the perform dissection. All
+        the queue based 'dissectors' will dissect one packet at a time, so
         it's better that this function is in the base class.
 
         """
         hdr, data = self.fetch()
 
+        if hdr == 'stop':
+            self._stop_requested = True
+            return hdr, data, None
+
         # FIXME: This might slowdown things but for now fine.
         if isinstance(hdr, PCAPHeader):
             newhdr = pcap_ffi.new('struct pcap_pkthdr *')
@@ -776,7 +855,7 @@ class WishpyDissectorQueuePython(WishpyDissectorQueue):
 
         self.__queue = queue
         self.__running = False
-        self.__stop_requested = False
+        self._stop_requested = False
         self._packets_fetched = 0
 
     def fetch(self):
@@ -810,13 +889,17 @@ class WishpyDissectorQueuePython(WishpyDissectorQueue):
             fetched += 1
             try:
                 hdr, data, d = self.dissect_one_packet()
+
+                if data and d is None:
+                    # Packet was there, but filter rejected it
+                    continue
             except:
                 _logger.exception("dissect_one_packet")
                 break
 
             x = yield (hdr, data, d)
 
-            if self.__stop_requested == True:
+            if self._stop_requested == True:
                 break
 
             if x and x.lower() == 'stop':
@@ -831,7 +914,7 @@ class WishpyDissectorQueuePython(WishpyDissectorQueue):
 
     def stop(self):
         """Stop's the generator by setting internal state."""
-        self.__stop_requested = True
+        self._stop_requested = True
 
 
 def setup_process():
@@ -856,8 +939,12 @@ def cleanup_process():
     Per process cleanup. de-init of epan/wtap modules.
     """
 
-    perform_epan_wtap_cleanup()
-
     global _EPAN_LIB_INITIALIZED
+
+    if _EPAN_LIB_INITIALIZED:
+        perform_epan_wtap_cleanup()
+    else:
+        _logger.warning("cleanup_process called without init process!")
+
     _EPAN_LIB_INITIALIZED = False
 
